module irq_source
  (   input         		Clk_i,             
      input         		Rstn_i,          
      input          		IRQSourceSChipSelect_i,    
      input [3:0]   		IRQSourceSAddress_i,       // 16-bytes reserved
      input         		IRQSourceSRead_i,          
      output [31:0]  		IRQSourceSReadData_o,      
      input         		IRQSourceSWrite_i,         
      input [31:0]   		IRQSourceSWriteData_i,     
      output         		IRQSourceSWaitRequest_o,  
      output 	    			Irq_o
    );
    
wire										request_in;
reg											request_reg;
wire										irq_read_ack;
wire										irq_write_ack;
wire										irq_ack;
wire		[31:0]					irq_read_data;
reg											irq_reg;


 
assign request_in = ((IRQSourceSRead_i | IRQSourceSWrite_i) & IRQSourceSChipSelect_i);

always @(posedge Clk_i or negedge Rstn_i)
  begin
  	if(~Rstn_i)
  	  request_reg <= 1'b0;
  	else
  	  request_reg <= request_in;
  end
 
  
assign irq_read_ack = ~request_reg & request_in & IRQSourceSRead_i;
assign irq_write_ack = ~request_reg & request_in & IRQSourceSWrite_i;
assign irq_ack = irq_read_ack | irq_write_ack;
  


// MSI-X structure Avalon-MM Slave interface output assignments
assign IRQSourceSWaitRequest_o = ~irq_ack;
assign IRQSourceSReadData_o    = (irq_read_ack)? irq_read_data : 32'h0; 


      always @(posedge Clk_i or negedge Rstn_i)
        begin
        	  if(~Rstn_i)
        	    irq_reg <= 1'b0;
        	  else if(irq_write_ack )
        	    irq_reg <= IRQSourceSWriteData_i ;
       	end
       	
	assign Irq_o = irq_reg;
	assign irq_read_data = {31'h0, irq_reg};
 	
endmodule
 	
 	