module pcie_irq 
		(
// Slave Inteface
			input        			Clk_i,             // MSI-X structure slave
			input       			Rstn_i,          
			input          		MsixStructSChipSelect_i,    
			input 		[13:0]  MsixStructSAddress_i,       // 16KB space
			input 		[3:0]  	MsixStructSByteEnable_i,    
			input         		MsixStructSRead_i,          
			output 		[31:0] 	MsixStructSReadData_o,      
			input         		MsixStructSWrite_i,         
			input			[31:0]	MsixStructSWriteData_i,     
			output        		MsixStructSWaitRequest_o,  
			
	// Master Interface
			output          	MsixMWrite_o,
			output 		[31:0]	MsixMAddress_o,
			output 		[31:0]	MsixMWriteData_o,
			output 		[3:0] 	MsixMByteEnable_o,
			input             MsixMWaitRequest_i,      
			
// IRQ interface               
			input			[3:0]		Irq_i,	                        
			
	// PCIe HIP interface
			input			[81:0]	MsiIntfc_i,
			input			[15:0]	MsixIntfc_i,
			output						IntxReq_o,
			input							IntxAck_i
		);
		
			wire			[31:0]	table_read_data;                              
			wire			[3:0]		pba_clear;                          
			wire			[3:0]		pba_set;                         
			wire			[3:0]		pba_q;
			wire			[4:0]		table_read_address;


// MSIX structure block
msix_struct msix_structure
(
			.Clk_i(Clk_i),            
			.Rstn_i(Rstn_i),                                        
			.MsixStructSChipSelect_i(MsixStructSChipSelect_i),                       
			.MsixStructSAddress_i(MsixStructSAddress_i),         
			.MsixStructSByteEnable_i(MsixStructSByteEnable_i),                       
			.MsixStructSRead_i(MsixStructSRead_i),                             
			.MsixStructSReadData_o(MsixStructSReadData_o),                         			
			.MsixStructSWrite_i(MsixStructSWrite_i),                            
			.MsixStructSWriteData_i(MsixStructSWriteData_i),                        	
			.MsixStructSWaitRequest_o(MsixStructSWaitRequest_o),                      	
			.TableReadAddress_i(table_read_address),                            
			.TableReadData_o(table_read_data),                               
			.SetPba_i(pba_set),                                   
			.ClearPba_i(pba_clear),                               
			.PbaData_o(pba_q)   
);


/// IRQ generator via Mwr
msix_trans_gen irq_gen
(
			.Clk_i(Clk_i),
			.Rstn_i(Rstn_i),
			.MsixMWrite_o(MsixMWrite_o),
			.MsixMAddress_o(MsixMAddress_o),
			.MsixMWriteData_o(MsixMWriteData_o),
			.MsixMByteEnable_o(MsixMByteEnable_o),
			.MsixMWaitRequest_i(MsixMWaitRequest_i),
			.TableReadAddress_o(table_read_address),    
			.TableReadData_i(table_read_data),       
			.SetPba_o(pba_set),    
			.ClearPba_o(pba_clear),
			.PbaData_i(pba_q),
			.Irq_i(Irq_i),	
			.MsiIntfc_i(MsiIntfc_i),
			.MsixIntfc_i(MsixIntfc_i),
			.IntxReq_o(IntxReq_o)
);               

endmodule              